@@ -17,15 +17,19 @@ limitations under the License.
 
 import datetime
 import json
-import logging
 import os
 import subprocess
 import tempfile
 import time
 
-from ConfigParser import ConfigParser
+from freezer.utils import utils
+from oslo_config import cfg
+from oslo_log import log
+from six.moves import configparser
 
-from freezer import utils
+
+CONF = cfg.CONF
+LOG = log.getLogger(__name__)
 
 
 class StopState(object):
@@ -68,7 +72,7 @@ class ScheduledState(object):
 
     @staticmethod
     def abort(job, doc):
-        return ScheduledState.stop(job, doc)
+        return StopState.stop(job, doc)
 
     @staticmethod
     def start(job, doc):
@@ -93,7 +97,8 @@ class RunningState(object):
     @staticmethod
     def abort(job, doc):
         job.event = Job.ABORT_EVENT
-        return Job.NO_EVENT
+        job.scheduler.update_job(job.id, job.job_doc)
+        return Job.ABORTED_RESULT
 
     @staticmethod
     def start(job, doc):
@@ -129,11 +134,13 @@ class Job(object):
     def create(scheduler, executable, job_doc):
         job = Job(scheduler, executable, job_doc)
         if job.job_doc_status in ['running', 'scheduled']:
-            logging.warning('Job {0} already has {1} status, '
-                            'skipping'.format(job.id, job.job_doc_status))
-            return None
-        if not job.event and not job.job_doc_status:
-            logging.warning('Autostart Job {0}'.format(job.id))
+            LOG.warning('Resetting {0} status from job {1}'
+                        .format(job.job_doc_status, job.id))
+        if job.job_doc_status == 'stop' and not job.event:
+            LOG.info('Job {0} was stopped.'.format(job.id))
+            job.event = Job.STOP_EVENT
+        elif not job.event:
+            LOG.info('Autostart Job {0}'.format(job.id))
             job.event = Job.START_EVENT
         return job
 
@@ -147,7 +154,7 @@ class Job(object):
     def remove(self):
         with self.scheduler.lock:
             # delegate to state object
-            logging.info('REMOVE job {0}'.format(self.id))
+            LOG.info('REMOVE job {0}'.format(self.id))
             self.state.remove(self)
 
     @property
@@ -199,10 +206,9 @@ class Job(object):
 
     @staticmethod
     def save_action_to_file(action, f):
-        parser = ConfigParser()
-        parser.add_section('action')
+        parser = configparser.ConfigParser()
         for action_k, action_v in action.items():
-            parser.set('action', action_k, action_v)
+            parser.set('DEFAULT', action_k, action_v)
         parser.write(f)
         f.seek(0)
 
@@ -214,6 +220,14 @@ class Job(object):
     def schedule_interval(self):
         return self.job_doc['job_schedule'].get('schedule_interval', '')
 
+    @property
+    def schedule_start_date(self):
+        return self.job_doc['job_schedule'].get('schedule_start_date', '')
+
+    @property
+    def schedule_end_date(self):
+        return self.job_doc['job_schedule'].get('schedule_end_date', '')
+
     @property
     def schedule_cron_fields(self):
         cron_fields = ['year', 'month', 'day', 'week', 'day_of_week',
@@ -227,15 +241,39 @@ class Job(object):
         return self.scheduler.is_scheduled(self.id)
 
     def get_schedule_args(self):
+        def get_start_date(date):
+            # start_date format "%Y-%m-%dT%H:%M:%S"
+            now = datetime.datetime.now()
+            start_date = now + datetime.timedelta(0, 2, 0)
+            if (utils.date_to_timestamp(date) >
+                    utils.date_to_timestamp(now.isoformat().split('.')[0])):
+                start_date = datetime.datetime.strptime(
+                    date, "%Y-%m-%dT%H:%M:%S")
+            return start_date
+
+        def get_end_date(start, end):
+            # start end format "%Y-%m-%dT%H:%M:%S"
+            end_date = datetime.datetime.strptime(end, "%Y-%m-%dT%H:%M:%S")
+            if (utils.date_to_timestamp(start) > utils.date_to_timestamp(end)):
+                end_date = None
+            return end_date
+        kwargs_date = {}
+        if self.schedule_start_date:
+            kwargs_date.update({
+                'start_date': get_start_date(self.schedule_start_date)
+            })
+            if self.schedule_end_date:
+                end_date = get_end_date(self.schedule_start_date,
+                                        self.schedule_end_date)
+                kwargs_date.update({
+                    'end_date': end_date
+                })
         if self.schedule_date:
             return {'trigger': 'date',
                     'run_date': self.schedule_date}
         elif self.schedule_interval:
             kwargs = {'trigger': 'interval'}
-            if not self.schedule_date:
-                kwargs.update({
-                    'start_date': datetime.datetime.now() +
-                    datetime.timedelta(0, 2, 0)})
+            kwargs.update(kwargs_date)
             if self.schedule_interval == 'continuous':
                 kwargs.update({'seconds': 1})
             else:
@@ -245,7 +283,8 @@ class Job(object):
         else:
             cron_fields = self.schedule_cron_fields
             if cron_fields:
-                return {'trigger': 'cron'}.update(cron_fields)
+                kwargs = {'trigger': 'cron'}.update(kwargs_date)
+                return kwargs.update(cron_fields)
         # no scheduling information, schedule to start within a few seconds
         return {'trigger': 'date',
                 'run_date': datetime.datetime.now() +
@@ -256,96 +295,139 @@ class Job(object):
             next_event = job_doc['job_schedule'].get('event', '')
             while next_event:
                 if next_event == Job.STOP_EVENT:
-                    logging.info('JOB {0} event: STOP'.format(self.id))
+                    LOG.info('JOB {0} event: STOP'.format(self.id))
                     next_event = self.state.stop(self, job_doc)
                 elif next_event == Job.START_EVENT:
-                    logging.info('JOB {0} event: START'.format(self.id))
+                    LOG.info('JOB {0} event: START'.format(self.id))
                     next_event = self.state.start(self, job_doc)
                 elif next_event == Job.ABORT_EVENT:
-                    logging.info('JOB {0} event: ABORT'.format(self.id))
+                    LOG.info('JOB {0} event: ABORT'.format(self.id))
                     next_event = self.state.abort(self, job_doc)
+                elif next_event == Job.ABORTED_RESULT:
+                    LOG.info('JOB {0} aborted.'.format(self.id))
+                    break
 
     def upload_metadata(self, metadata_string):
         try:
             metadata = json.loads(metadata_string)
             if metadata:
+                metadata['job_id'] = self.id
                 self.scheduler.upload_metadata(metadata)
-                logging.info("[*] Job {0}, freezer action metadata uploaded"
-                             .format(self.id))
+                LOG.info("Job {0}, freezer action metadata uploaded"
+                         .format(self.id))
         except Exception as e:
-            logging.error('[*] metrics upload error: {0}'.format(e))
+            LOG.error('metrics upload error: {0}'.format(e))
 
     def execute_job_action(self, job_action):
-        max_retries = job_action.get('max_retries', 1)
-        tries = max_retries
+        max_tries = (job_action.get('max_retries', 0) + 1)
+        tries = max_tries
         freezer_action = job_action.get('freezer_action', {})
         max_retries_interval = job_action.get('max_retries_interval', 60)
         action_name = freezer_action.get('action', '')
         config_file_name = None
         while tries:
-
             with tempfile.NamedTemporaryFile(delete=False) as config_file:
                 self.save_action_to_file(freezer_action, config_file)
                 config_file_name = config_file.name
-                freezer_command = '{0} --metadata-out - --config {1}'.\
+                freezer_command = '{0} --metadata-out - --config-file {1}'.\
                     format(self.executable, config_file.name)
                 self.process = subprocess.Popen(freezer_command.split(),
                                                 stdout=subprocess.PIPE,
                                                 stderr=subprocess.PIPE,
                                                 env=os.environ.copy())
+
+                # store the pid for this process in the api
+                try:
+                    self.job_doc['job_schedule']['current_pid'] = \
+                        self.process.pid
+
+                    self.scheduler.update_job(self.job_doc['job_id'],
+                                              self.job_doc)
+                except Exception as error:
+                    LOG.error("Error saving the process id {}".format(error))
+
                 output, error = self.process.communicate()
                 # ensure the tempfile gets deleted
                 utils.delete_file(config_file_name)
 
             if error:
-                logging.error("[*] Freezer client error: {0}".format(error))
+                LOG.error("Freezer client error: {0}".format(error))
             elif output:
                 self.upload_metadata(output)
 
-            if self.process.returncode:
+            if self.process.returncode == -15:
+                # This means the job action was aborted by the scheduler
+                LOG.warning('Freezer-agent was killed by the scheduler. '
+                            'Cleanup should be done manually: container, '
+                            'mountpoint and lvm snapshots.')
+                return Job.ABORTED_RESULT
+
+            elif self.process.returncode:
                 # ERROR
                 tries -= 1
                 if tries:
-                    logging.warning('[*] Job {0} failed {1} action,'
-                                    ' retrying in {2} seconds'
-                                    .format(self.id, action_name,
-                                            max_retries_interval))
+                    LOG.warning('Job {0} failed {1} action,'
+                                ' retrying in {2} seconds'
+                                .format(self.id, action_name,
+                                        max_retries_interval))
                     # sleeping with the bloody lock, but we don't want other
                     # actions to mess with our stuff like fs snapshots, do we ?
                     time.sleep(max_retries_interval)
             else:
                 # SUCCESS
-                logging.info('[*] Job {0} action {1}'
-                             ' returned success exit code'.
-                             format(self.id, action_name))
+                LOG.info('Job {0} action {1}'
+                         ' returned success exit code'.
+                         format(self.id, action_name))
                 return Job.SUCCESS_RESULT
-        logging.error('[*] Job {0} action {1} failed after {2} tries'
-                      .format(self.id, action_name, max_retries))
+        LOG.error('Job {0} action {1} failed after {2} tries'
+                  .format(self.id, action_name, max_tries))
 
         return Job.FAIL_RESULT
 
+    def contains_exec(self):
+        jobs = self.job_doc.get('job_actions')
+        for job in jobs:
+            freezer_action = job.get('freezer_action')
+            action = freezer_action.get('action')
+            if action == 'exec':
+                return True
+        return False
+
     def execute(self):
         result = Job.SUCCESS_RESULT
         with self.scheduler.execution_lock:
             with self.scheduler.lock:
-                logging.info('job {0} running'.format(self.id))
+                LOG.info('job {0} running'.format(self.id))
                 self.state = RunningState
                 self.job_doc_status = Job.RUNNING_STATUS
                 self.scheduler.update_job_status(self.id, self.job_doc_status)
 
             self.start_session()
+            # if the job contains exec action and the scheduler passes the
+            # parameter --disable-exec job execution should fail
+            if self.contains_exec() and CONF.disable_exec:
+                LOG.info("Job {0} failed because it contains exec action "
+                         "and exec actions are disabled by scheduler"
+                         .format(self.id))
+                self.result = Job.FAIL_RESULT
+                self.finish()
+                return
 
             for job_action in self.job_doc.get('job_actions', []):
                 if job_action.get('mandatory', False) or\
                         (result == Job.SUCCESS_RESULT):
+
                     action_result = self.execute_job_action(job_action)
                     if action_result == Job.FAIL_RESULT:
                         result = Job.FAIL_RESULT
+
+                    if action_result == Job.ABORTED_RESULT:
+                        result = Job.ABORTED_RESULT
                 else:
                     freezer_action = job_action.get('freezer_action', {})
                     action_name = freezer_action.get('action', '')
-                    logging.warning("[*]skipping {0} action".
-                                    format(action_name))
+                    LOG.warning("skipping {0} action".
+                                format(action_name))
             self.result = result
             self.finish()
 
@@ -385,12 +467,12 @@ class Job(object):
                     self.session_tag = resp['session_tag']
                     return
             except Exception as e:
-                logging.error('[*]Error while starting session {0}. {1}'.
-                              format(self.session_id, e))
-            logging.warning('[*]Retrying to start session {0}'.
-                            format(self.session_id))
+                LOG.error('Error while starting session {0}. {1}'.
+                          format(self.session_id, e))
+            LOG.warning('Retrying to start session {0}'.
+                        format(self.session_id))
             retry -= 1
-        logging.error('[*]Unable to start session {0}'.format(self.session_id))
+        LOG.error('Unable to start session {0}'.format(self.session_id))
 
     def end_session(self, result):
         if not self.session_id:
@@ -405,22 +487,24 @@ class Job(object):
                 if resp['result'] == 'success':
                     return
             except Exception as e:
-                logging.error('[*]Error while ending session {0}. {1}'.
-                              format(self.session_id, e))
-            logging.warning('[*]Retrying to end session {0}'.
-                            format(self.session_id))
+                LOG.error('Error while ending session {0}. {1}'.
+                          format(self.session_id, e))
+            LOG.warning('Retrying to end session {0}'.
+                        format(self.session_id))
             retry -= 1
-        logging.error('[*]Unable to end session {0}'.format(self.session_id))
+        LOG.error('Unable to end session {0}'.format(self.session_id))
 
     def schedule(self):
         try:
             kwargs = self.get_schedule_args()
-            self.scheduler.add_job(self.execute, id=self.id, **kwargs)
+            self.scheduler.add_job(self.execute, id=self.id,
+                                   executor='threadpool',
+                                   misfire_grace_time=3600, **kwargs)
         except Exception as e:
-            logging.error("[*] Unable to schedule job {0}: {1}".
-                          format(self.id, e))
+            LOG.error("Unable to schedule job {0}: {1}".
+                      format(self.id, e))
 
-        logging.info('scheduler job with parameters {0}'.format(kwargs))
+        LOG.info('scheduler job with parameters {0}'.format(kwargs))
 
         if self.scheduled:
             self.job_doc_status = Job.SCHEDULED_STATUS
